from selenium.webdriver.support.wait import WebDriverWait

from simon.accounts.locators import LoginLocators


class CheckBoxException(Exception):
    pass


class RememberMeCheckBox(object):
    locator = None

    def __set__(self, instance, value: bool):
        """Sets the state of the checkbox."""
        if not self._is_boolean(value):
            raise CheckBoxException("You are working with a checkbox: only booleans accepted.")
        driver = instance.driver
        WebDriverWait(driver, 100).until(
            lambda driver: driver.find_element(*self.locator))
        checkbox = driver.find_element(*self.locator)
        self._set_checkbox(checkbox, value)

    def __get__(self, instance, owner):
        """Gets the current state of the checkbox."""
        driver = instance.driver
        WebDriverWait(driver, 100).until(
            lambda driver: driver.find_element(*self.locator))
        return driver.find_element(*self.locator).is_selected()

    @staticmethod
    def _is_boolean(value):
        if (value is True) or (value is False):
            return True

    @staticmethod
    def _set_checkbox(checkbox, value):
        """
        avoiding to do anything is the wanted stated
        of the checkbox is the current value.
        """
        if value is True and not checkbox.is_selected():
            checkbox.click()
        if value is False and checkbox.is_selected():
            checkbox.click()


class LoginRememberMeCheckBox(RememberMeCheckBox):
    locator = LoginLocators.REMEMBER_ME_CHECKBOX



from simon.chat.elements import ChatMessages, MessageWriter

from simon.chat.locators import ChatLocators
from simon.pages import BasePage


class ChatPage(BasePage):
    def __init__(self, driver):
        super().__init__(driver)
        self.messages = ChatMessages(driver)
        self.writer = MessageWriter(driver)

    def refresh(self):
        self.messages = ChatMessages(self.driver)

    def is_chat_new(self):
        msgs = self.messages.all()
        unread_msgs = self.messages.unread()
        if msgs and unread_msgs:
            if len(msgs) == len(unread_msgs):
                # I could check that the msg is not myself.
                return True

    # Header
    @property
    def contact_name(self):
        return self._find_element(ChatLocators.CHAT_HEADER_CONTACT_NAME).text

    @property
    def contact_status(self):
        return self._find_element(ChatLocators.CHAT_HEADER_CONTACT_STATUS).text

    @property
    def icon(self):
        return self._find_element(ChatLocators.CHAT_HEADER_CONTACT_ICON).get_attribute("src")

    def is_contact_online(self):
        status = self._find_element(ChatLocators.CHAT_HEADER_CONTACT_STATUS).text
        if status and status == "online":
            return True

    def is_contact_typing(self):
        # Not yet tested when the status is Typing...
        statuses = ["yping"]
        status = self._find_element(ChatLocators.CHAT_HEADER_CONTACT_STATUS)
        if status and (status in statuses):
            return True

    @property
    def menu(self):
        return self._find_element(ChatLocators.CHAT_HEADER_MENU)

    @property
    def attach(self):
        return self._find_element(ChatLocators.CHAT_HEADER_ATTACH)

    @property
    def search(self):
        return self._find_element(ChatLocators.CHAT_HEADER_SEARCH)

    # Messages
    @property
    def arrow_button(self):
        return self._find_element(ChatLocators.CHAT_BODY_ARROW_BUTTON)

    @property
    def arrow_button_notification_qty(self):
        if self.is_arrow_button_on_screen():
            qty = self._find_element(ChatLocators.CHAT_BODY_ARROW_BUTTON_NOTIFICATION_QTY).text
            return int(qty)

    def is_arrow_button_on_screen(self):
        if self.arrow_button:
            return True

    def is_notification_in_arrow_button(self):
        if self.arrow_button:
            if self.arrow_button_notification_qty:
                return True



from simon.chats.elements import OpenedChats

from simon.pages import BasePage


class PanePage(BasePage):
    opened_chats = OpenedChats()

    def get_first_opened_chat_with_notifications(self):
        for chat in self.opened_chats:
            if chat.has_notifications():
                return chat

    def get_opened_chat(self, name):
        for opened_chat in self.opened_chats:
            if name == opened_chat.name:
                return opened_chat



import datetime
import time
from typing import List

from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver import ActionChains
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.wait import WebDriverWait

from simon.chat.locators import ChatLocators


class Message:
    def __init__(self, element=None, driver=None):
        self.element = element
        self.driver = driver

    def find_element(self, locator):
        """
        Using this for all attrs ensures we are bound to the
        context of the message only(html code of the message only)
        avoiding grabbing other element in the whole html page.
        """
        try:
            # TODO: test the idea of using the self.driver this deep
            #       to refresh DOM elements in the self.element
            if self.element:
                return self.element.find_element(*locator)
        except NoSuchElementException:
            return None

    @property
    def contact(self):
        return self.__get_cleaned_contact()

    @property
    def date(self):
        return self.__get_cleaned_datetime()

    @property
    def text(self):
        if self.find_element(ChatLocators.CHAT_BODY_MSG_TEXT):
            return self.find_element(ChatLocators.CHAT_BODY_MSG_TEXT).text

    @property
    def status(self):
        if self.find_element(ChatLocators.CHAT_BODY_MSG_STATUS):
            return self.find_element(ChatLocators.CHAT_BODY_MSG_STATUS).get_attribute("aria-label").strip()

    def __get_cleaned_contact(self):
        _str = self.raw_datetime_and_contact()
        if _str:
            contact_str = _str.strip().rsplit(" ", 1)[1]
            return contact_str.replace(":", "")

    def __get_cleaned_datetime(self):
        _str = self.raw_datetime_and_contact()
        if _str:
            datetime_str = _str.strip().rsplit(" ", 1)[0]
            return datetime.datetime.strptime(datetime_str, "[%H:%M, %m/%d/%Y]")

    def raw_datetime_and_contact(self):
        if self.find_element(ChatLocators.CHAT_BODY_MSG_CONTACT_AND_DATETIME):
            # "[19:18, 1/29/2019] CmrH: "
            return self.find_element(
                ChatLocators.CHAT_BODY_MSG_CONTACT_AND_DATETIME
            ).get_attribute("data-pre-plain-text")

    def reply(self, msg: str):
        """Only works to reply msgs from the contact.
        DO NOT USE IT to reply your own msg. (Different css selectors needed)"""
        if msg:
            # Hover near by the reply arrow menu
            msg_text = self.find_element(ChatLocators.CHAT_BODY_MSG_TEXT)
            hover = ActionChains(self.driver).move_to_element(msg_text)
            hover.perform()

            # -> BOUND OUT OF html element context and the whole html page
            #    MAYBE in the base page context.
            # get arrow and click on it
            msg_arrow_menu = self.driver.find_element(*ChatLocators.CHAT_BODY_MSG_ARROW)
            msg_arrow_menu.click()

            reply_button = self.driver.find_element(*ChatLocators.CHAT_BODY_MSG_ARROW_POP_MENU_REPLY)
            reply_button.click()

            writer = MessageWriter(self.driver)
            writer.send_msg(msg)
            # <- END OUT OF BOUND
            return "Coders on break!"

    def __repr__(self):
        return f"Message({self.element})"

    def __str__(self):
        return f"{self.contact}: '({self.text})'"


class OnlyFilterByContact(Exception):
    pass


class ChatMessages:
    child_class = Message

    def __init__(self, driver):
        self.driver = driver

    def all(self, filterby: str = None):
        locator = self.get_correct_locator(filterby)
        order_by_oldest = [self.child_class(e, self.driver) for e in self.__find_elements(locator)]
        order_by_newest = order_by_oldest[::-1]
        return order_by_newest

    def get_correct_locator(self, filterby):
        if not filterby:
            return ChatLocators.CHAT_BODY_MSGS
        if filterby == 'contact':
            return ChatLocators.CHAT_BODY_MSGS_CONTACT
        if filterby == 'myself':
            return ChatLocators.CHAT_BODY_MSGS_MYSELF
        else:
            raise OnlyFilterByContact("For now we just can filter by contact name")

    def newest(self, qty: int = None, filterby=None):
        msgs = self.all(filterby)
        return self.__get_correct_qty(qty, msgs)

    def oldest(self, qty: int = None, filterby=None):
        msgs = self.all(filterby)[::-1]
        return self.__get_correct_qty(qty, msgs)

    def unread(self):
        qty = self.unread_qty()
        if qty:
            unread_msgs_by_newest = self.all(filterby='contact')[:qty]
            unread_msgs_by_oldest = unread_msgs_by_newest[::-1]
            return unread_msgs_by_oldest
        return []

    def unread_qty(self):
        if self.__find_element(ChatLocators.CHAT_BODY_UNREAD_MESSAGE):
            text = self.__find_element(ChatLocators.CHAT_BODY_UNREAD_MESSAGE).text
            # "4182 UNREAD MESSAGES"
            qty = text.split(" ", 1)[0]
            return int(qty)

    def __find_elements(self, locator):
        # TODO: THIS DEEP i do not have to worry about the real time
        #       state of the messages. It should hangle upper level.
        #       OR YES I DO? ANALYSE WHEN SEEN!
        WebDriverWait(self.driver, 100).until(
            lambda driver: self.driver.find_elements(*locator))
        elements = self.driver.find_elements(*locator)
        return [e for e in elements]

    def __find_element(self, locator):
        try:
            WebDriverWait(self.driver, 100).until(
                lambda driver: self.driver.find_element(*locator))
            return self.driver.find_element(*locator)
        except NoSuchElementException:
            return None

    def __get_correct_qty(self, qty, msgs):
        if not qty:
            return msgs[0]
        if self._have_enough(qty, len(msgs)):
            return msgs[:qty]
        else:
            return msgs

    @staticmethod
    def _have_enough(qty_asked: int, qty_msgs: int):
        if qty_asked <= qty_msgs:
            return True


class MessageWriter(object):
    def __init__(self, driver):
        self.driver = driver

    def send_msg(self, msg: str):
        text_input = self.__find_element(ChatLocators.CHAT_FOOTER_TEXT_INPUT_FIELD)
        if text_input and msg:
            text_input.send_keys(msg)
            time.sleep(1)
            text_input.send_keys(Keys.RETURN)

    def send_multiline_msg(self, msgs: List[str]):
        text_input = self.__find_element(ChatLocators.CHAT_FOOTER_TEXT_INPUT_FIELD)
        if text_input and msgs:
            time.sleep(1)
            for msg in msgs:
                text_input.send_keys(msg)
                # press SHIFT + ENTER (for new line)
                ActionChains(self.driver).key_down(Keys.SHIFT).send_keys(Keys.RETURN).key_up(Keys.SHIFT).perform()
            text_input.send_keys(Keys.RETURN)

    def send_msg_animated(self, msg, groove=0.22):
        text_input = self.__find_element(ChatLocators.CHAT_FOOTER_TEXT_INPUT_FIELD)
        for char in msg:
            text_input.send_keys(char)
            time.sleep(groove)
        text_input.send_keys(Keys.RETURN)

    def __find_element(self, locator):
        try:
            WebDriverWait(self.driver, 100).until(
                lambda driver: self.driver.find_element(*locator))
            return self.driver.find_element(*locator)
        except NoSuchElementException:
            return None



from selenium.webdriver.common.by import By


class SearchLocators(object):
    SEARCH = (By.CSS_SELECTOR, "._2EoyP")
    # CHATS = (,)
    # GROUPS = (,)
    # CONTACTS = (,)  # creating a new chat
    # MESSAGES = (,)



from selenium.webdriver.common.by import By


class HeaderLocators(object):
    HEADER = (By.CSS_SELECTOR, "._1QUKR")

    HEADER_PROFILE = (By.CSS_SELECTOR, "._1MXsz ._3Whw5")
    HEADER_STATUS = (By.CSS_SELECTOR, ".PVMjB:nth-child(1)")
    HEADER_NEW_CHAT = (By.CSS_SELECTOR, ".PVMjB:nth-child(2)")

    HEADER_MENU = (By.CSS_SELECTOR, ".PVMjB:nth-child(3)")
    HEADER_MENU_POP_MENU = (By.CSS_SELECTOR, ".PVMjB:nth-child(3) .I4jbF")
    HEADER_MENU_POP_MENU_CREATE_GROUP = (By.CSS_SELECTOR, ".PVMjB:nth-child(3) .I4jbF li:nth-child(1) div")
    HEADER_MENU_POP_MENU_CREATE_ROOM = (By.CSS_SELECTOR, ".PVMjB:nth-child(3) .I4jbF li:nth-child(2) div")
    HEADER_MENU_POP_MENU_PROFILE = (By.CSS_SELECTOR, ".PVMjB:nth-child(3) .I4jbF li:nth-child(3) div")
    HEADER_MENU_POP_MENU_ARCHIVED = (By.CSS_SELECTOR, ".PVMjB:nth-child(3) .I4jbF li:nth-child(4) div")
    HEADER_MENU_POP_MENU_STARRED = (By.CSS_SELECTOR, ".PVMjB:nth-child(3) .I4jbF li:nth-child(5) div")
    HEADER_MENU_POP_MENU_SETTINGS = (By.CSS_SELECTOR, ".PVMjB:nth-child(3) .I4jbF li:nth-child(6) div")
    HEADER_MENU_POP_MENU_LOGOUT = (By.CSS_SELECTOR, ".PVMjB:nth-child(3) .I4jbF li:nth-child(7) div")



from selenium.webdriver.common.by import By


class ChatLocators(object):
    CHAT = (By.CSS_SELECTOR, "#main")

    # CHAT INFO & DETAILS, CHAT MENU & SETTINGS
    CHAT_HEADER = (By.CSS_SELECTOR, "#main header")
    CHAT_HEADER_CONTACT_ICON = (By.CSS_SELECTOR, "#main header ._2goTk")  # attr:(src)
    CHAT_HEADER_CONTACT_NAME = (By.CSS_SELECTOR, "#main header ._3ko75")
    CHAT_HEADER_CONTACT_STATUS = (By.CSS_SELECTOR, "#main header ._3-cMa")
    CHAT_HEADER_MENU = (By.CSS_SELECTOR, "#main header .PVMjB:nth-child(3) div")
    CHAT_HEADER_ATTACH = (By.CSS_SELECTOR, "#main header .PVMjB:nth-child(2) div")
    CHAT_HEADER_SEARCH = (By.CSS_SELECTOR, "#main header .PVMjB:nth-child(1) div")

    # READING MSGS
    CHAT_BODY = (By.CSS_SELECTOR, "#main ._2-aNW .z_tTQ")
    # all
    CHAT_BODY_MSGS = (By.CSS_SELECTOR, "._2hqOq.message-in, ._2hqOq.message-out")
    # filtered by "contact"
    CHAT_BODY_MSGS_MYSELF = (By.CSS_SELECTOR, "._2hqOq.message-out")
    CHAT_BODY_MSGS_CONTACT = (By.CSS_SELECTOR, "._2hqOq.message-in")
    # filtered by "type"
    CHAT_BODY_MSGS_TYPE_VOICE_RECORDING = (By.CSS_SELECTOR, "._3D5PJ")
    CHAT_BODY_MSGS_TYPE_VIDEO = (By.CSS_SELECTOR, ".S9d8c")
    CHAT_BODY_MSGS_TYPE_IMAGE = (By.CSS_SELECTOR, ".xOg_4")
    CHAT_BODY_MSGS_TYPE_GIFF = (By.CSS_SELECTOR, "._3gBQS")
    CHAT_BODY_MSGS_TYPE_AUDIO = (By.CSS_SELECTOR, "")

    # message details
    CHAT_BODY_MSG_CONTACT_AND_DATETIME = (By.CSS_SELECTOR, "._3sKvP.wQZ0F ._274yw div.copyable-text")
    CHAT_BODY_MSG_STATUS = (By.CSS_SELECTOR, "._3sKvP.wQZ0F ._274yw div._2frDn ._1qPwk span")
    CHAT_BODY_MSG_TEXT = (By.CSS_SELECTOR, "._3sKvP.wQZ0F ._274yw div.copyable-text ._3Whw5 span")
    CHAT_BODY_MSG_ARROW_MY_OWN_MSG = (By.CSS_SELECTOR, "._4tndQ._1q11a ._2oA--")
    CHAT_BODY_MSG_ARROW = (By.CSS_SELECTOR, "._2nBjH._1q11a ._2oA--")

    # -> THIS ARE OUT OF THE CONTEXT OF THE HTML ELEMENT.
    CHAT_BODY_MSG_ARROW_POP_MENU = (By.CSS_SELECTOR, "._2s_eZ")
    CHAT_BODY_MSG_ARROW_POP_MENU_REPLY = (By.CSS_SELECTOR, "._2s_eZ ._1N-3y.eP_pD._36Osw:nth-child(1) div")
    CHAT_BODY_MSG_ARROW_POP_MENU_FORWARD = (By.CSS_SELECTOR, "._2s_eZ ._1N-3y.eP_pD._36Osw:nth-child(2) div")
    CHAT_BODY_MSG_ARROW_POP_MENU_STAR = (By.CSS_SELECTOR, "._2s_eZ ._1N-3y.eP_pD._36Osw:nth-child(3) div")
    CHAT_BODY_MSG_ARROW_POP_MENU_DELETE = (By.CSS_SELECTOR, "._2s_eZ ._1N-3y.eP_pD._36Osw:nth-child(4) div")
    CHAT_BODY_MSG_ARROW_POP_MENU_MY_OWN_MSG_MSG_INFO = (By.CSS_SELECTOR, "._2s_eZ ._1N-3y.eP_pD._36Osw:nth-child(1) div")
    CHAT_BODY_MSG_ARROW_POP_MENU_MY_OWN_MSG_REPLY = (By.CSS_SELECTOR, "._2s_eZ ._1N-3y.eP_pD._36Osw:nth-child(2) div")
    CHAT_BODY_MSG_ARROW_POP_MENU_MY_OWN_MSG_FORWARD = (By.CSS_SELECTOR, "._2s_eZ ._1N-3y.eP_pD._36Osw:nth-child(3) div")
    CHAT_BODY_MSG_ARROW_POP_MENU_MY_OWN_MSG_STAR = (By.CSS_SELECTOR, "._2s_eZ ._1N-3y.eP_pD._36Osw:nth-child(4) div")
    CHAT_BODY_MSG_ARROW_POP_MENU_MY_OWN_MSG_DELETE = (By.CSS_SELECTOR, "._2s_eZ ._1N-3y.eP_pD._36Osw:nth-child(5) div")
    # <- END
    CHAT_BODY_UNREAD_MESSAGE = (By.CSS_SELECTOR, "._9WQEN")
    CHAT_BODY_ARROW_BUTTON = (By.CSS_SELECTOR, "._1YcH-._1-MYr")
    CHAT_BODY_ARROW_BUTTON_NOTIFICATION_QTY = (By.CSS_SELECTOR, "._31gEB")

    # SENDING MSGS
    CHAT_FOOTER = (By.CSS_SELECTOR, "#main footer")
    CHAT_FOOTER_SMILEY_ICON = (By.CSS_SELECTOR, "#main footer ._2X5R7")
    CHAT_FOOTER_TEXT_INPUT_FIELD = (By.CSS_SELECTOR, "#main footer ._3FRCZ")
    CHAT_FOOTER_RECORD_ICON = (By.CSS_SELECTOR, "#main footer ._2r1fJ")



## 1st
# can read a message's text, time, arrow, type
# can reply to a (specific) message
## 2nd
# can read most recent message
# can read 5 most recent messages
# can read oldest message available
# can read 5 oldest messages available
# can iterate over all messages (default=most recent)
## 3rd
# can read last unread messages
# can iterate over last unread messages
## 4th
# can detect if current chat is fresh (new) chat initiated by contact/client
## 5th
# can write and send a message
# can send a multiline message
## 6th
# can search for a message (by: text, time(approx, range), type)
## optional
# can send a link
# can send an image
# can send a video
# can send an audio
# can record a voice from microphone
#######################################################
import datetime
import time
import unittest
import uuid as uuid

from simon.chat.pages import ChatPage
from simon.chats.pages import PanePage
from simon.pages import safe_path
from simon.tests.base import LoggedInTestCase, FileBaseTestCase


class ChatPageTests(FileBaseTestCase):
    driver = None
    file_path = "/home/fantaso/github/el_simon/simon/tests/html/rastri_chat_template_full.html"

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.chat_page = ChatPage(cls.driver)
        cls.chat_page.load(safe_path(cls.file_path))

    def test_whatsapp_title_matches(self):
        self.assertIn("WhatsApp", self.driver.title)

    # HEADER
    def test_can_check_contact_chat_status_and_contact_info(self):
        # TODO: provide html templates for distinct cases
        self.assertEqual(self.chat_page.contact_name, "Rastri")
        self.assertEqual(self.chat_page.contact_status, "online")
        icon_file = "file:///home/fantaso/github/el_simon/simon/tests/html/rastri_chat_template_full_files/pp.jpeg"
        self.assertEqual(self.chat_page.icon, icon_file)
        self.assertTrue(self.chat_page.is_contact_online())
        self.assertFalse(self.chat_page.is_contact_typing())

    def test_can_get_chat_tools(self):
        # vague test for now as I dont need this functionality implemented
        # but at least making sure the css selectors are correct
        self.assertEqual(self.chat_page.menu.get_attribute("title"), "Menu")
        self.assertEqual(self.chat_page.attach.get_attribute("title"), "Attach")
        self.assertEqual(self.chat_page.search.get_attribute("title"), "Search…")

    # SEARCHER
    # # can search for a message (by: text, time(approx, range), type)
    # msgs = chat_page.messages.search("a msg that contains some keywords")
    # msgs = chat_page.messages.search_type('video')  # audio, voice recording, link, text, status
    # msgs = chat_page.messages.search_date(datetime.datetime)  # a range of 5 up or down the date
    # msgs = chat_page.messages.search_date(datetime.datetime, datetime.datetime)  # ramge
    # msgs = chat_page.messages.search_last_hour()  # day, week, month, year (order by newest)
    # msgs = chat_page.messages.search_by_hour(4)  # day, week, month, year (order by newest)

    # MESSAGES
    def test_can_read_a_message_details_querying_with_newest(self):
        msg = self.chat_page.messages.newest()
        self.assertEqual(msg.contact, "Rastri")
        self.assertEqual(msg.date, datetime.datetime(2020, 7, 8, 20, 33, 00))
        self.assertEqual(msg.text, "Lslsls")
        self.assertIsNone(msg.status)  # this message does not have arrow status

    def test_can_read_unread_messages_and_iterate_over_them(self):
        unread_msgs = self.chat_page.messages.unread()

        self.assertEqual(len(unread_msgs), 4)
        # starts at first unread (oldest msg unread)
        self.assertEqual(unread_msgs[0].text, "Isksk")
        self.assertEqual(unread_msgs[1].text, "Jdjs")
        self.assertEqual(unread_msgs[2].text, "Jsjss")
        self.assertEqual(unread_msgs[3].text, "Lslsls")
        # finishes at last unread (newest msg unread)
        for msg in unread_msgs:
            self.assertEqual(msg.contact, "Rastri")
            self.assertEqual(msg.date, datetime.datetime(2020, 7, 8, 20, 33, 00))
            self.assertIsNone(msg.status)  # these msgs dont have arrow statuses

    def test_can_read_most_recent_message(self):
        msg = self.chat_page.messages.newest()
        self.assertEqual(msg.contact, "Rastri")
        self.assertEqual(msg.date, datetime.datetime(2020, 7, 8, 20, 33, 00))
        self.assertEqual(msg.text, "Lslsls")

    def test_can_read_five_most_recent_messages(self):
        msgs = self.chat_page.messages.newest(5)
        self.assertEqual(len(msgs), 5)

        my_msg = msgs[4]
        self.assertEqual(my_msg.text, "Verga loco.. se me borraron 4 manda 4 más ahí.")
        self.assertEqual(my_msg.date, datetime.datetime(2020, 7, 8, 20, 28, 00))
        self.assertEqual(my_msg.contact, "CmrH")
        self.assertIn(my_msg.status, "Delivered")

        # contact
        contact_msgs = msgs[:4]
        self.assertEqual(len(contact_msgs), 4)
        for msg in contact_msgs:
            self.assertEqual(msg.contact, "Rastri")
            self.assertEqual(msg.date, datetime.datetime(2020, 7, 8, 20, 33, 00))
            self.assertIsNone(msg.status)  # this message does not have arrow status

    def test_can_read_oldest_message(self):
        msg = self.chat_page.messages.oldest()
        self.assertEqual(msg.contact, "Rastri")
        self.assertEqual(msg.date, datetime.datetime(2019, 1, 29, 12, 46, 00))
        self.assertEqual(msg.text, "nena aun en vzla?")
        self.assertIsNone(msg.status)  # this message does not have arrow status

    def test_can_read_five_oldest_messages(self):
        msgs = self.chat_page.messages.oldest(5)
        self.assertEqual(len(msgs), 5)

        contact_msg = msgs[0]
        self.assertEqual(contact_msg.text, "nena aun en vzla?")
        self.assertEqual(contact_msg.date, datetime.datetime(2019, 1, 29, 12, 46, 00))
        self.assertEqual(contact_msg.contact, "Rastri")
        self.assertIsNone(contact_msg.status)

        contact_msg = msgs[1]
        self.assertEqual(contact_msg.text, "Mi amor")
        self.assertEqual(contact_msg.date, datetime.datetime(2019, 1, 29, 18, 55, 00))
        self.assertEqual(contact_msg.contact, "CmrH")
        self.assertEqual(contact_msg.status, "Delivered")

        # contact
        my_msgs = msgs[1:]
        self.assertEqual(len(my_msgs), 4)
        for msg in my_msgs:
            self.assertEqual(msg.contact, "CmrH")
            self.assertEqual(msg.status, "Delivered")
            # the time of these 4 messages are between 18:55 and 18:56
            self.assertGreaterEqual(msg.date, datetime.datetime(2019, 1, 29, 18, 55, 00))
            self.assertLessEqual(msg.date, datetime.datetime(2019, 1, 29, 18, 56, 00))

    def test_can_get_all_messages_at_once(self):
        msgs = self.chat_page.messages.all()
        self.assertEqual(len(msgs), 2030)
        # make sure its being ordered by default (newest)

        newest_msg = msgs[0]
        self.assertEqual(newest_msg.date, datetime.datetime(2020, 7, 8, 20, 33, 00))
        self.assertEqual(newest_msg.text, "Lslsls")
        self.assertEqual(newest_msg.contact, "Rastri")

        oldest_msg = msgs[-1]
        self.assertEqual(oldest_msg.date, datetime.datetime(2019, 1, 29, 12, 46, 00))
        self.assertEqual(oldest_msg.text, "nena aun en vzla?")
        self.assertEqual(oldest_msg.contact, "Rastri")

        # SOME objects which are none do not posses a datetime available
        # comparing_date = datetime.datetime(2019, 1, 1, 00, 00, 00)
        # for msg in msgs:
        #     self.assertGreaterEqual(msg.date, comparing_date)
        #     comparing_date = msg.date

    def test_can_read_most_recent_message_from_contact(self):
        msg = self.chat_page.messages.newest(filterby='contact')
        self.assertEqual(msg.contact, "Rastri")
        self.assertEqual(msg.text, "Lslsls")
        self.assertEqual(msg.date, datetime.datetime(2020, 7, 8, 20, 33, 00))

    def test_can_read_oldest_message_from_contact(self):
        msg = self.chat_page.messages.oldest(filterby='contact')
        self.assertEqual(msg.contact, "Rastri")
        self.assertEqual(msg.text, "nena aun en vzla?")
        self.assertEqual(msg.date, datetime.datetime(2019, 1, 29, 12, 46, 00))

    def test_can_read_most_recent_message_from_myself(self):
        msg = self.chat_page.messages.newest(filterby='myself')
        self.assertEqual(msg.contact, "CmrH")
        self.assertEqual(msg.text, "Verga loco.. se me borraron 4 manda 4 más ahí.")
        self.assertEqual(msg.date, datetime.datetime(2020, 7, 8, 20, 28, 00))

    def test_can_read_oldest_message_from_myself(self):
        msg = self.chat_page.messages.oldest(filterby='myself')
        self.assertEqual(msg.contact, "CmrH")
        self.assertEqual(msg.text, "Mi amor")
        self.assertEqual(msg.date, datetime.datetime(2019, 1, 29, 18, 55, 00))

    def test_can_read_ten_msgs_order_by_most_recent_from_contact(self):
        msgs = self.chat_page.messages.newest(10, filterby='contact')

        self.assertEqual(len(msgs), 10)

        newest = msgs[0]
        fifth = msgs[4]
        tenth = msgs[9]

        self.assertEqual(newest.contact, "Rastri")
        self.assertEqual(newest.text, "Lslsls")
        self.assertEqual(newest.date, datetime.datetime(2020, 7, 8, 20, 33, 00))

        self.assertEqual(fifth.contact, "Rastri")
        self.assertEqual(fifth.text, "Loca")
        self.assertEqual(fifth.date, datetime.datetime(2020, 7, 8, 20, 25, 00))

        self.assertEqual(tenth.contact, "Rastri")
        self.assertEqual(tenth.text, "Ha ok")
        self.assertEqual(tenth.date, datetime.datetime(2020, 7, 7, 18, 11, 00))

    def test_can_read_ten_msgs_order_by_oldest_from_contact(self):
        msgs = self.chat_page.messages.oldest(10, filterby='contact')

        self.assertEqual(len(msgs), 10)

        oldest = msgs[0]
        third = msgs[2]
        fourth = msgs[3]

        self.assertEqual(oldest.contact, "Rastri")
        self.assertEqual(oldest.text, "nena aun en vzla?")
        self.assertEqual(oldest.date, datetime.datetime(2019, 1, 29, 12, 46, 00))

        self.assertEqual(third.contact, "Rastri")
        self.assertEqual(third.text, "como viste la cosa")
        self.assertEqual(third.date, datetime.datetime(2019, 1, 29, 19, 15, 00))

        self.assertEqual(fourth.contact, "Rastri")
        self.assertEqual(fourth.text, "a que loco")
        self.assertEqual(fourth.date, datetime.datetime(2019, 1, 29, 19, 19, 00))

    def test_can_ten_all_msgs_order_by_most_recent_from_myself(self):
        msgs = self.chat_page.messages.newest(10, filterby='myself')

        self.assertEqual(len(msgs), 10)

        newest = msgs[0]
        fourth = msgs[3]
        seventh = msgs[6]

        self.assertEqual(newest.contact, "CmrH")
        self.assertEqual(newest.text, "Verga loco.. se me borraron 4 manda 4 más ahí.")
        self.assertEqual(newest.date, datetime.datetime(2020, 7, 8, 20, 28, 00))

        self.assertEqual(fourth.contact, "CmrH")
        self.assertEqual(fourth.text, "Ya la cosa.")
        self.assertEqual(fourth.date, datetime.datetime(2020, 7, 7, 16, 47, 00))

        self.assertEqual(seventh.contact, "CmrH")
        self.assertEqual(seventh.text, "Jajajaja")
        self.assertEqual(seventh.date, datetime.datetime(2020, 7, 6, 15, 1, 00))

    def test_can_read_all_msgs_order_by_oldest_from_myself(self):
        msgs = self.chat_page.messages.oldest(10, filterby='myself')

        self.assertEqual(len(msgs), 10)

        oldest = msgs[0]
        sixth = msgs[5]
        ninth = msgs[8]

        self.assertEqual(oldest.contact, "CmrH")
        self.assertEqual(oldest.text, "Mi amor")
        self.assertEqual(oldest.date, datetime.datetime(2019, 1, 29, 18, 55, 00))

        self.assertEqual(sixth.contact, "CmrH")
        self.assertEqual(sixth.text, "Jjaa")
        self.assertEqual(sixth.date, datetime.datetime(2019, 1, 29, 19, 18, 00))

        self.assertEqual(ninth.contact, "CmrH")
        self.assertEqual(ninth.text, "Si vaca..")
        self.assertEqual(ninth.date, datetime.datetime(2019, 1, 30, 18, 6, 00))

    def test_can_read_all_msgs_order_by_most_recent_from_contact(self):
        msgs = self.chat_page.messages.all(filterby='contact')
        self.assertEqual(len(msgs), 868)
        msg = msgs[0]
        self.assertEqual(msg.text, "Lslsls")

    def test_can_read_all_msgs_order_by_most_recent_from_myself(self):
        msgs = self.chat_page.messages.all(filterby='myself')
        self.assertEqual(len(msgs), 1162)
        msg = msgs[0]
        self.assertEqual(msg.text, "Verga loco.. se me borraron 4 manda 4 más ahí.")

    def test_can_detect_if_chat_is_completely_new(self):
        # can detect if current chat is fresh (new) chat initiated by contact/client
        self.assertFalse(self.chat_page.is_chat_new())

    def test_can_read_messages_notifications_in_arrow_in_go_to_bottom_button(self):
        # with current html template can not test a negative case (if button is not available)
        self.assertIsNotNone(self.chat_page.arrow_button,
                             "The chat page attribute should return an element (selenium object)")
        self.assertTrue(self.chat_page.is_arrow_button_on_screen(),
                        "Not detecting the arrow button on the chat screen")
        self.assertTrue(self.chat_page.is_notification_in_arrow_button(),
                        "Not detecting the amount of new messages notifications on the arrow bottom on the chat screen")
        self.assertEqual(self.chat_page.arrow_button_notification_qty, 4,
                         "Not detecting the 4 new messages notifications on the arrow bottom on the chat screen")


class InteractiveBaseTestCase(LoggedInTestCase):
    WAIT_TIME = 2
    OPENED_CHAT_CONTACT_NAME = "+58 412-7624636"
    SUCCESSFUL_MSG_STATUS = ["Sent", "Delivered"]

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        pane_page = PanePage(cls.driver)
        pane_page.get_opened_chat(cls.OPENED_CHAT_CONTACT_NAME).click()
        cls.chat_page = ChatPage(cls.driver)


class InteractiveWriterTests(InteractiveBaseTestCase):

    def test_writer_can_send_a_message_in_the_chat(self):
        writer = self.chat_page.writer
        uid = str(uuid.uuid4())
        writer.send_msg(f"Testing sending a unique msg. {uid}")

        # small time for the message to be "Delivered" or "Sent" instead "Pending"
        time.sleep(self.WAIT_TIME)
        msg = self.chat_page.messages.newest()
        self.assertEqual(msg.text, f"Testing sending a unique msg. {uid}")
        self.assertIn(msg.status, self.SUCCESSFUL_MSG_STATUS)
        self.assertNotEqual(msg.text, "Failed")

    def test_writer_can_send_a_multiline_message_in_the_chat(self):
        multiline_msg = ["Line # 1", "Line # 2"]
        writer = self.chat_page.writer
        writer.send_multiline_msg(multiline_msg)

        # small time for the message to be "Delivered" or "Sent" instead "Pending"
        time.sleep(self.WAIT_TIME)
        msg = self.chat_page.messages.newest()
        self.assertEqual(msg.text, "Line # 1\nLine # 2")
        self.assertIn(msg.status, self.SUCCESSFUL_MSG_STATUS)
        self.assertNotEqual(msg.text, "Failed")

    def test_writer_can_send_a_message_in_the_chat_animated(self):
        writer = self.chat_page.writer
        uid = str(uuid.uuid4())
        writer.send_msg_animated(f"Testing sending a unique animated msg. {uid}",
                                 groove=0.25)

        # small time for the message to be "Delivered" or "Sent" instead "Pending"
        time.sleep(self.WAIT_TIME)
        msg = self.chat_page.messages.newest()
        self.assertEqual(msg.text, f"Testing sending a unique animated msg. {uid}")
        self.assertIn(msg.status, self.SUCCESSFUL_MSG_STATUS)
        self.assertNotEqual(msg.text, "Failed")


class InteractiveTests(InteractiveBaseTestCase):

    def test_can_reply_to_a_specific_message(self):
        """
        Only working to reply any msg that is coming from your contact.
        To reply a msg you send yourself, you need to use diff css selectors and so on.
        """
        # Test that it can reply just to text messages.
        # voice or videos is diffrernt for the css selector

        msg = self.chat_page.messages.newest(filterby='contact')
        uid = str(uuid.uuid4())
        msg.reply(f"Testing a unique reply to a specific msg. {uid} filterby by contact.")

        # small time for the message to be "Delivered" or "Sent" instead "Pending"
        time.sleep(self.WAIT_TIME)
        # TODO: IMPORTANT.. getting the newest() does not ensure the msg to reply will be from your contact
        #       as messages returns all messages found in the chat
        new_msg = self.chat_page.messages.newest()
        self.assertEqual(new_msg.text, f"Testing a unique reply to a specific msg. {uid} filterby by contact.")


if __name__ == "__main__":
    unittest.main()



import os
import pathlib
import time

from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.support.wait import WebDriverWait

from .chat.locators import ChatLocators
from .chats.locators import PaneLocators
from .header.locators import HeaderLocators
from .locators import WelcomeLocators
from .search.locators import SearchLocators


def element_not_found(func):
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except NoSuchElementException:
            return False

    return wrapper


def safe_path(abs_path):
    path = os.path.abspath(abs_path)
    url = pathlib.Path(path).as_uri()
    # url = "file:///home/$USER/project_folder/html/test_template.html"
    return url


class BasePage(object):
    def __init__(self, driver):
        self.driver = driver

    def is_title_matches(self):
        return "WhatsApp" in self.driver.title

    def load(self, url=None):
        if not url:
            self.driver.get("https://web.whatsapp.com/")
        else:
            self.driver.get(url)
        time.sleep(2.5)

    def refresh(self):
        self.driver.refresh()

    @element_not_found
    def is_welcome_page_available(self):
        if self._find_element(WelcomeLocators.WELCOME):
            return True

    @element_not_found
    def is_nav_bar_page_available(self):
        if self._find_element(HeaderLocators.HEADER):
            return True

    @element_not_found
    def is_search_page_available(self):
        if self._find_element(SearchLocators.SEARCH):
            return True

    @element_not_found
    def is_pane_page_available(self):
        if self._find_element(PaneLocators.PANE):
            return True

    @element_not_found
    def is_chat_page_available(self):
        if self._find_element(ChatLocators.CHAT):
            return True

    def _find_element(self, locator):
        try:
            WebDriverWait(self.driver, 100).until(
                lambda driver: self.driver.find_element(*locator))
            return self.driver.find_element(*locator)
        except NoSuchElementException:
            return None


class WelcomePage(BasePage):
    def is_welcome_message_matches(self):
        return "Keep your phone connected" in self.driver.page_source



from simon.accounts.elements import LoginRememberMeCheckBox

from simon.pages import BasePage


class LoginPage(BasePage):
    remember_me = LoginRememberMeCheckBox()

    def is_instruction_title_matches(self):
        return "To use WhatsApp on your computer" in self.driver.page_source

    def is_remember_me_selected(self):
        return self.remember_me is True



import unittest

from selenium.webdriver.common.keys import Keys

from simon.chat.locators import ChatLocators  # TODO: this ref should not be coupled with another page
from simon.chats.pages import PanePage
from simon.tests.base import LoggedInTestCase


class PanePageTests(LoggedInTestCase):
    driver = None

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.pane_page = PanePage(cls.driver)
        cls.pane_page.load()

    def test_can_get_opened_chats(self):
        self.assertGreaterEqual(len(self.pane_page.opened_chats), 5)

    def test_can_click_in_the_first_opened_chat(self):
        first_chat = self.pane_page.opened_chats[0]
        first_chat.click()
        self.assertIsNotNone(self.driver.find_element(*ChatLocators.CHAT))

    def test_can_get_first_opened_chat_details(self):
        first_chat = self.pane_page.opened_chats[0]
        self.assertIsNotNone(first_chat.name)
        self.assertIsNotNone(first_chat.icon)
        self.assertIsNotNone(first_chat.last_message)
        self.assertIsNotNone(first_chat.last_message_time)

    def test_can_iterate_over_first_five_opened_chats(self):
        for opened_chat in self.pane_page.opened_chats[:5]:
            self.assertIsNotNone(opened_chat.name)
            self.assertIsNotNone(opened_chat.icon)
            self.assertIsNotNone(opened_chat.last_message)
            self.assertIsNotNone(opened_chat.last_message_time)

    def test_can_detect_first_opened_chat_with_new_messages(self):
        chat = self.pane_page.get_first_opened_chat_with_notifications()
        self.assertIsInstance(chat.notifications, int)
        self.assertGreaterEqual(chat.notifications, 1)

    def test_can_detect_first_opened_chat_with_new_messages_and_click_it(self):
        chat = self.pane_page.get_first_opened_chat_with_notifications()
        self.assertGreaterEqual(chat.notifications, 1)
        chat.click()
        self.assertIsNotNone(self.driver.find_element(*ChatLocators.CHAT))
        self.assertIn("unread message", self.driver.page_source.lower())

    def test_can_detect_first_opened_chat_with_new_messages_and_send_a_message(self):
        """
        This a manual test. It will be removed when
        the elements are created for the ChatPage.
        """
        chat = self.pane_page.get_first_opened_chat_with_notifications()
        chat.click()
        chat_text_field = self.driver.find_element(*ChatLocators.CHAT_FOOTER_TEXT_INPUT_FIELD)
        chat_text_field.send_keys("Hi, I am Simon: Carlos's WhatsApp ChatBot!")
        chat_text_field.send_keys(Keys.RETURN)


if __name__ == "__main__":
    unittest.main()



import time

from selenium.common.exceptions import NoSuchElementException, StaleElementReferenceException
from selenium.webdriver.support.wait import WebDriverWait

from simon.chats.locators import PaneLocators


class ReadOnlyBaseElement(object):
    def __init__(self, element):
        self.element = element

    def __get__(self, instance, owner):
        return self.element

    def __set__(self, instance, value):
        raise Exception("Can not set a value to a read-only element")

    def __repr__(self):
        return f"<ReadOnlyBaseElement: {self.element}>"


class ReadOnlyBaseElements(object):
    locator = None
    child_class = None

    def __get__(self, instance, owner):
        """Gets the current state of the element."""
        driver = instance.driver
        WebDriverWait(driver, 100).until(
            lambda driver: driver.find_elements(*self.locator))
        elements = driver.find_elements(*self.locator)
        return [self.child_class(ele) for ele in elements]

    def __set__(self, instance, values):
        raise Exception("Can not set a value to a read-only elements")


def element_not_found(func):
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except NoSuchElementException:
            return ""
        except StaleElementReferenceException:
            return ""

    return wrapper


class Chat(ReadOnlyBaseElement):
    @property
    @element_not_found
    def name(self):
        return self.element.find_element(*PaneLocators.NAME).text

    @property
    @element_not_found
    def last_message_time(self):
        return self.element.find_element(*PaneLocators.LAST_MESSAGE_TIME).text

    @property
    @element_not_found
    def last_message(self):
        return self.element.find_element(*PaneLocators.LAST_MESSAGE).text

    @property
    @element_not_found
    def icon(self):
        return self.element.find_element(*PaneLocators.ICON).get_attribute('src')

    @property
    @element_not_found
    def notifications(self):
        return int(self.element.find_element(*PaneLocators.NOTIFICATION).text)

    @element_not_found
    def has_notifications(self):
        notifications = int(self.element.find_element(*PaneLocators.NOTIFICATION).text)
        if notifications > 0:
            return True

    @element_not_found
    def click(self):
        self.element.click()
        time.sleep(3)


class OpenedChats(ReadOnlyBaseElements):
    locator = PaneLocators.OPENED_CHATS
    child_class = Chat



from selenium.webdriver.common.by import By


class PaneLocators(object):
    # parent
    PANE = (By.CSS_SELECTOR, "#pane-side")

    # childs
    OPENED_CHATS = (By.CSS_SELECTOR, "#pane-side ._210SC")

    # grand child of each child
    ## left side
    ICON = (By.CSS_SELECTOR, "._325lp img")  # attr(src)
    ## right side
    ### upper side
    NAME = (By.CSS_SELECTOR, "._2kHpK ._3ko75._5h6Y_")
    LAST_MESSAGE_TIME = (By.CSS_SELECTOR, "._2kHpK .m61XR")
    ### bottom side
    # ARROW_STATUS = (By.CSS_SELECTOR, "._210SC ._1582E .zFnXi")
    LAST_MESSAGE = (By.CSS_SELECTOR, "._2kHpK ._1582E ._3Whw5")
    NOTIFICATION = (By.CSS_SELECTOR, "._210SC .m61XR ._31gEB")



import time
from unittest import TestCase

from selenium import webdriver

from simon.accounts.pages import LoginPage
from simon.header.pages import HeaderPage


class TearDownClass:
    driver = None


class LoggedInTestCase(TearDownClass, TestCase):
    driver = None

    @classmethod
    def setUpClass(cls):
        cls.driver = webdriver.Firefox()
        cls.driver.maximize_window()

        # Manually login in
        cls.login_page = LoginPage(cls.driver)
        cls.login_page.load()
        time.sleep(7)

    @classmethod
    def tearDownClass(cls):
        header_page = HeaderPage(cls.driver)
        header_page.logout()
        cls.driver.quit()


class FileBaseTestCase(TearDownClass, TestCase):
    driver = None

    @classmethod
    def setUpClass(cls):
        cls.driver = webdriver.Firefox()
        cls.driver.maximize_window()

    @classmethod
    def tearDownClass(cls):
        cls.driver.quit()



import time
import unittest
from unittest import TestCase

from selenium import webdriver

from simon.accounts.pages import LoginPage
from simon.header.pages import HeaderPage
from simon.pages import BasePage


class RegistrationBaseTestCase(TestCase):
    def setUp(self):
        self.driver = webdriver.Firefox()
        self.driver.maximize_window()
        self.login_page = LoginPage(self.driver)
        self.login_page.load()

    def tearDown(self):
        self.driver.close()


class LoginPageTests(RegistrationBaseTestCase):
    def test_can_open_whatsapp_login_page(self):
        self.assertTrue(self.login_page.is_title_matches())
        self.assertTrue(self.login_page.is_instruction_title_matches())

    def test_checkbox_remember_me_is_checked_by_default(self):
        self.assertTrue(self.login_page.is_remember_me_selected())

    def test_can_uncheck_checkbox_remember_me(self):
        self.login_page.remember_me = False
        self.assertFalse(self.login_page.is_remember_me_selected())

    def test_can_uncheck_and_check_again_checkbox_remember_me(self):
        self.login_page.remember_me = False
        self.assertFalse(self.login_page.is_remember_me_selected())
        self.login_page.remember_me = True
        self.assertTrue(self.login_page.is_remember_me_selected())

    def test_can_manually_login_successfully(self):
        base_page = BasePage(self.driver)
        base_page.load()
        # time for you to read QR code and access whatsapp
        time.sleep(8)
        self.assertTrue(base_page.is_title_matches())
        self.assertTrue(base_page.is_welcome_page_available())
        self.assertTrue(base_page.is_nav_bar_page_available())
        self.assertTrue(base_page.is_search_page_available())
        self.assertTrue(base_page.is_pane_page_available())
        # chat is only available after you click on a person to open the chat
        self.assertFalse(base_page.is_chat_page_available())


class LogoutTest(RegistrationBaseTestCase):
    def test_can_logout_successfully_after_login(self):
        header_page = HeaderPage(self.driver)
        # time for you to read QR code and access whatsapp
        time.sleep(8)
        self.assertTrue(header_page.is_welcome_page_available())
        self.assertTrue(header_page.is_nav_bar_page_available())

        header_page.logout()
        self.assertTrue(self.login_page.is_title_matches())
        self.assertTrue(self.login_page.is_instruction_title_matches())


if __name__ == "__main__":
    unittest.main()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="whatsapp-web",
    version="0.0.1",
    author="fantaso",
    author_email="fantaso.code@gmail.com",
    description="A Python library to automate easily whatsapp web",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Fantaso/whatsapp-web",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment :: Mozilla",
        "Natural Language :: English",
        "Topic :: Communications :: Chat",
    ],
    python_requires='>=3.6',
)



from simon.header.locators import HeaderLocators
from simon.pages import BasePage


class HeaderPage(BasePage):

    @property
    def profile(self):
        return self._find_element(HeaderLocators.HEADER_PROFILE)

    @property
    def status(self):
        return self._find_element(HeaderLocators.HEADER_STATUS)

    @property
    def new_chat(self):
        return self._find_element(HeaderLocators.HEADER_NEW_CHAT)

    @property
    def menu(self):
        return self._find_element(HeaderLocators.HEADER_MENU)

    @property
    def menu_create_group(self):
        return self._find_element(HeaderLocators.HEADER_MENU_POP_MENU_CREATE_GROUP)

    @property
    def menu_create_room(self):
        return self._find_element(HeaderLocators.HEADER_MENU_POP_MENU_CREATE_ROOM)

    @property
    def menu_profile(self):
        return self._find_element(HeaderLocators.HEADER_MENU_POP_MENU_PROFILE)

    @property
    def menu_archived(self):
        return self._find_element(HeaderLocators.HEADER_MENU_POP_MENU_ARCHIVED)

    @property
    def menu_starred(self):
        return self._find_element(HeaderLocators.HEADER_MENU_POP_MENU_STARRED)

    @property
    def menu_settings(self):
        return self._find_element(HeaderLocators.HEADER_MENU_POP_MENU_SETTINGS)

    @property
    def menu_logout(self):
        return self._find_element(HeaderLocators.HEADER_MENU_POP_MENU_LOGOUT)

    def logout(self):
        self.menu.click()
        self.menu_logout.click()

    def is_profile_icon_available(self):
        if self.profile:
            return True

    def is_status_icon_available(self):
        if self.status:
            return True

    def is_new_chat_icon_available(self):
        if self.new_chat:
            return True

    def is_menu_icon_available(self):
        if self._find_element(HeaderLocators.HEADER_MENU):
            return True
